#include <FLAC/format.h>
#include <FLAC++/encoder.h>
#include <FLAC++/metadata.h>

#include <iostream>

class FlacFile : public FLAC::Encoder::File {
public:
	void test_c_api() {
		set_filename("/tmp/test-c.flac");

		set_streamable_subset(false);

		set_channels(1);

		set_bits_per_sample(16);

		set_sample_rate(32000);

		set_blocksize(65535);

		/* C API struct */
		FLAC__StreamMetadata ** meta = new FLAC__StreamMetadata*[1];

		FLAC::Metadata::VorbisComment * vc = new FLAC::Metadata::VorbisComment;

		FLAC::Metadata::VorbisComment::Entry entry;
		entry.set_field_name("hello");
		entry.set_field_value("world");
		vc->append_comment(entry);

		meta[0] = const_cast< ::FLAC__StreamMetadata *>(static_cast<const ::FLAC__StreamMetadata *>(*vc));

		/* This uses the method that simply passes the parameter
		 * directly to the C API.
		 * This does not fail.
		 */
		set_metadata(meta, 1);

		/* Deleting either vc or meta here is also a problem. */

		init();

		/* Meta and vc can be deleted from now on. */

		finish();

		delete vc;
		delete [] meta;
	}

	void test_cpp_api() {
		set_filename("/tmp/test-cpp.flac");

		set_streamable_subset(false);

		set_channels(1);

		set_sample_rate(32000);

		set_blocksize(65535);
		
		/* C++ API object */
		FLAC::Metadata::Prototype ** meta = new FLAC::Metadata::Prototype*[1];

		FLAC::Metadata::VorbisComment * vc = new FLAC::Metadata::VorbisComment;

		FLAC::Metadata::VorbisComment::Entry entry;
		entry.set_field_name("hello");
		entry.set_field_value("world");
		vc->append_comment(entry);

		meta[0] = vc;

		/* This calls the broken C++ API method which temporarily
		 * converts the C++ object array meta to an array of C structs.
		 * Unpatched this causes a failure in init().
		 */
		set_metadata(meta, 1);

		/* Deleting vc here would be a problem.
		 * It is probably safe (but unwise) to delete meta.
		 */

		init();

		/* Meta and vc can be deleted from now on. */

		finish();

		delete vc;
		delete [] meta;
	}
	
};

int main(int argc, char ** argv) {
	FlacFile f1, f2;

	std::cout << "Using C API -- this should not fail" << std::endl;
	f1.test_c_api();

	std::cout << "Using C++ API -- this should fail on FLAC 1.1.2" << std::endl;
	f2.test_cpp_api();

	std::cout << "FLAC has been fixed -- no failure." << std::endl;

	return 0;
}

